pub struct Layers<F: Field, T: Layer<F>> {
pub layers: Vec<T>,
marker: PhantomData<F>,
}Expand description
The list of Layers that make up the GKR circuit
Fields
layers: Vec<T>A Vec of pointers to various layer types
marker: PhantomData<F>Implementations
Sourceimpl<F: Field, T: Layer<F>> Layers<F, T>
impl<F: Field, T: Layer<F>> Layers<F, T>
Sourcepub fn add_gate(
&mut self,
nonzero_gates: Vec<(u32, u32, u32)>,
lhs: DenseMle<F>,
rhs: DenseMle<F>,
num_dataparallel_bits: Option<usize>,
gate_operation: BinaryOperation,
) -> DenseMle<F>
pub fn add_gate( &mut self, nonzero_gates: Vec<(u32, u32, u32)>, lhs: DenseMle<F>, rhs: DenseMle<F>, num_dataparallel_bits: Option<usize>, gate_operation: BinaryOperation, ) -> DenseMle<F>
Add a batched Add Gate layer to a list of layers
In the batched case, consider a vector of mles corresponding to an mle for each “batch” or “copy”.
Add a Gate layer to a list of layers
In the batched case (num_dataparallel_bits > 0), consider a vector of mles corresponding to an mle for each “batch” or “copy”.
Then we refer to the mle that represents the concatenation of these mles by interleaving as the
flattened mle and each individual mle as a batched mle.
Arguments
nonzero_gates: the gate wiring between single-copy circuit (as the wiring for each circuit remains the same)
x is the label on the batched mle lhs, y is the label on the batched mle rhs, and z is the label on the next layer, batched
lhs: the flattened mle representing the left side of the summationrhs: the flattened mle representing the right side of the summationnum_dataparallel_bits: the number of bits representing the circuit copy we are looking atgate_operation: which operation the gate is performing. right now, can either be an ‘add’ or ‘mul’ gate
Returns
A flattened DenseMle that represents the evaluations of the add gate wiring on lhs and rhs over the boolean hypercube
Sourcepub fn new_with_layers(layers: Vec<T>) -> Self
pub fn new_with_layers(layers: Vec<T>) -> Self
Creates a new Layers struct with populated layers values.
Sourcepub fn num_layers(&self) -> usize
pub fn num_layers(&self) -> usize
Returns the number of layers in the GKR circuit